﻿using System.IO;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Xml;
using System.Xml.Serialization;

namespace XmlSign
{
	public static class XmlSign
	{
		private static string Sign(string signString, string privateKey)
		{
			XmlDocument xmlDocument = new XmlDocument();
			xmlDocument.LoadXml(signString);

			CspParameters cspParams = new CspParameters { KeyContainerName = "XML_DSIG_RSA_KEY" };

			RSACryptoServiceProvider rsaCryptoServiceProvider = new RSACryptoServiceProvider(cspParams);
			rsaCryptoServiceProvider.FromXmlString(privateKey);

			SignedXml signedXml = new SignedXml(xmlDocument) { SigningKey = rsaCryptoServiceProvider };

			Reference reference = new Reference { Uri = "" };

			XmlDsigEnvelopedSignatureTransform env = new XmlDsigEnvelopedSignatureTransform();
			reference.AddTransform(env);

			signedXml.AddReference(reference);

			signedXml.ComputeSignature();

			XmlElement xmlDigitalSignature = signedXml.GetXml();

			xmlDocument.DocumentElement.AppendChild(xmlDocument.ImportNode(xmlDigitalSignature, true));

			return xmlDocument.OuterXml;
		}

		public static string Sign(XmlDocument xmlDocument, string privateKey)
		{
			return Sign(xmlDocument.OuterXml, privateKey);
		}

		public static string Sign<T>(T signClass, string privateKey)
		{
			XmlSerializer ser = new XmlSerializer(typeof(T));
			StringWriter streamWriter = new StringWriter();
			ser.Serialize(streamWriter, signClass);

			return Sign(streamWriter.ToString(), privateKey);
		}

		public static bool Verify(string xmlInput, string publicKey)
		{
			XmlDocument xmlDocument = new XmlDocument();
			xmlDocument.LoadXml(xmlInput);
			SignedXml signedXml = new SignedXml(xmlDocument);

			XmlNodeList nodeList = xmlDocument.GetElementsByTagName("Signature");
			// Load the first <signature> node.  
			signedXml.LoadXml((XmlElement)nodeList[0]);

			RSA rsa = RSA.Create();
			rsa.FromXmlString(publicKey);

			// Check the signature and return the result.));
			bool verified = signedXml.CheckSignature(rsa);

			return verified;
		}

		public static bool Verify<T>(string xmlInput, string publicKey, out T resultObject)
		{
			bool verified = Verify(xmlInput, publicKey);
			if (verified)
			{
				XmlSerializer ser = new XmlSerializer(typeof (T));
				StringReader stringReader = new StringReader(xmlInput);

				resultObject = (T) ser.Deserialize(stringReader);
				return verified;
			}

			resultObject = default(T);
			return false;
		}
	}
}
